"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkBashRequirements = exports.templateExists = exports.getTemplateLocation = exports.remove = exports.add = void 0;
var child_process_1 = require("child_process");
var fs_extra_1 = __importDefault(require("fs-extra"));
var path_1 = __importDefault(require("path"));
var errors_1 = require("./errors");
var logUtils_1 = require("./logUtils");
var paths_1 = require("./paths");
function add(templateName, pathOrUrl, remote) {
    var error = console.error;
    if (isReservedName(templateName)) {
        error("The template name '" + templateName + "' looks like a reserved name");
        error("You cannot create a template with this name.");
        error("Please retry with another name");
        throw new Error();
    }
    if (remote)
        return createTemplateFromRemote(pathOrUrl, templateName);
    createTemplateFromLocal(pathOrUrl, templateName);
}
exports.add = add;
function createTemplateFromLocal(source, templateName) {
    var debug = console.debug, error = console.error;
    var pathExists = fs_extra_1.default.existsSync(source);
    if (!pathExists) {
        throw new Error("The provided source doesn't exist.");
    }
    var destination = path_1.default.resolve(paths_1.TEMPLATES, templateName);
    debug("Destination folder :");
    debug(destination);
    debug();
    var templateExists = fs_extra_1.default.existsSync(destination);
    if (templateExists) {
        debug("Path " + destination + " already exists!");
        debug();
        error("This template name is already used. You can:");
        error("  - Delete the existing template");
        error("  - Choose another name");
        error();
        throw new Error("The template already exists !");
    }
    debug("Copying '" + source + "'");
    debug("To: '" + destination + "'");
    debug();
    fs_extra_1.default.copySync(source, destination);
}
function createTemplateFromRemote(source, templateName) {
    var debug = console.debug;
    var destination = path_1.default.join(paths_1.TEMPLATES, templateName);
    var command = "git clone \"" + source + "\" \"" + destination + "\"";
    try {
        debug("Running \"" + command + "\"");
        child_process_1.execSync(command);
    }
    catch (err) {
        throw new errors_1.OrigeenError("Unkown error while executing commmand", [
            "Check that 'git' is installed on your machine",
            "Make sure that '" + templateName + "' is not already used for another template",
        ]);
    }
    fs_extra_1.default.rmdirSync(path_1.default.join(destination, ".git"), { recursive: true });
}
function remove(templateName) {
    var error = console.error, debug = console.debug;
    if (isReservedName(templateName)) {
        error("You gave a template name beginning with '@'");
        error("Those are reserved template names and thus cannot be deleted");
        error();
        throw new Error("Naming error");
    }
    var pathToTemplate = path_1.default.join(paths_1.TEMPLATES, templateName);
    if (!fs_extra_1.default.existsSync(pathToTemplate)) {
        error("The given template name was not found.");
        error("Make sure you spelled it properly");
        error();
        throw new Error("Template not found");
    }
    debug("Removing '" + pathToTemplate + "' recursively");
    debug();
    fs_extra_1.default.rmdirSync(pathToTemplate, { recursive: true });
}
exports.remove = remove;
function isReservedName(name) {
    if (name.startsWith("@"))
        return true;
    return false;
}
function getTemplateLocation(templateName) {
    return path_1.default.join(paths_1.TEMPLATES, templateName);
}
exports.getTemplateLocation = getTemplateLocation;
function templateExists(templateName) {
    return fs_extra_1.default.existsSync(getTemplateLocation(templateName));
}
exports.templateExists = templateExists;
function checkBashRequirements(templateName) {
    try {
        var file = fs_extra_1.default.readFileSync(path_1.default.join(paths_1.TEMPLATES, templateName, "bashRequirements.txt"), { encoding: "utf-8" });
        var lines = file.split(/\r?\n/) || "";
        lines.forEach(function (line) {
            if (!line || line.startsWith("#") || line === "")
                return;
            var word = line.split(" ")[0];
            try {
                child_process_1.execSync(line);
            }
            catch (err) {
                throw new errors_1.OrigeenError("The template you are using requires that some cli utilities are installed.\n" +
                    ("Origeen tried to run " + logUtils_1.bash(line, false) + " but it didn't work.\n\nIt looks like you are missing a requirement."), [
                    "Run " + logUtils_1.bash(line, false) + " manually",
                    "Install " + logUtils_1.bash(word, false) + " and make sure to restart the terminal",
                ]);
            }
        });
    }
    catch (err) {
        return;
    }
}
exports.checkBashRequirements = checkBashRequirements;
